## Data generation
n = 500
# generate the real covariate
# Assume later that the precsion of x, namely 1, is known.
# On log-scale the initial value will hence be fixed to 0.
x = rnorm(n)
# generate the observed covariate, i.e. real one plus error
xobs = x + rnorm(n, sd =2)
# generate the data using the real (but unobservable) covariate
y = 1 + 3*x + rnorm(n, sd = 0.5)

# check linear models
lm(y ~ x)
lm(y ~ xobs)
# => Using xobs we observe attenuation of the real effect

## In INLA:

## I. naive way: estimate assuming there is no error
formula = y~xobs
model = inla(formula, family="gaussian", data=list(y=y, x=x, xobs=xobs))
print(model$summary.fixed)

# compare with result using correct (but unobserved) covariate
formula = y~x
model = inla(formula, family="gaussian", data=list(y=y, x=x, xobs=xobs))
print(model$summary.fixed)

## II. Plug-in approach
## 1. Estimate x
idx = 1:n
data = list(y=y, x=x, xobs=xobs, idx=idx)

formula2 = xobs ~ f(idx, model="iid", hyper=list(
	prec=list(
		initial=0, fixed=TRUE)))

result2 = inla(formula2, family="gaussian", data=data)

# save the posterior mean of x as new (better?) covariate
xest = result2$summary.random$idx$mean

## 2. Plug-in estimate
data = list(y=y, x=x, xobs=xobs, idx=idx, xest=xest)

formula2b = y ~ xest

result2b = inla(formula2b, family="gaussian", data=data)

print(result2b$summary.fixed)

## III. Joint model: to propagate uncertainty in xest

intercept = c(rep(1, n), rep(NA, n))
idx1 = c(1:n, rep(NA,n))
idx2 = c(rep(NA,n), 1:n)

Y = matrix(NA, 2*n, 2)
Y[1:n,1] = y
Y[(n+1):(2*n),2] = xobs

formulaJ = Y ~ -1 + intercept +
	f(idx2, model="iid", hyper=list(
			prec=list(initial=0, 
					  fixed=TRUE))) +
	f(idx1, copy="idx2", hyper=list(
		beta=list(
				fixed=FALSE,
				prior="gaussian",
				param=c(0,0.1))))

modelJ = inla(formulaJ, family=c("gaussian", "gaussian"),
	control.family=list(
		list(),
		list(
			hyper=list(
				prec=list(initial=log(1/2^2), fixed=FALSE)))),
	data=list(Y=Y, idx1=idx1, idx2=idx2, intercept=intercept))

summary(modelJ)
# Caution: The result for beta is now saved in the hyperparameters
print(modelJ$summary.hyperpar["Beta for idx1",])

